## Inspiré de http://python-prepa.github.io/information_theory.html
## Les messages seront donnés sous forme de chaine de caractères

def table_frequences(texte):
    """renvoie un dictionnaire avec le nombre d'occurences pour chacun des différents caractères rencontrés
    Parameters :
    ------------ texte est un str 
    Returns:
    -------- un dictionnaire avec les caractères en clé et les nombres d'occurence en valeurs
    table_frequences('abracadabra') renvoie {'c': 1, 'a': 5, 'd': 1, 'r': 2, 'b': 2}
    """
    table = {}
    for caractere in texte:
        if caractere in table:
            table[caractere] = table[caractere] + 1
        else:
            table[caractere] = 1
    return table

#fonction arbre_huffman qui étant donné un dictionnaire construit par la fonction précédente retourne une représentation de l’arbre de Huffman correspondant.
#
#Le prototype de la fonction sera le suivant def arbre_huffman (occurrences):
#et l’appel de arbre_huffman({'c': 1, 'a': 5, 'd': 1, 'r': 2, 'b': 2})
#devra retourner un dictionnaire de la forme {0: 'a', 1: {0: 'r', 1: {0: {0: 'c', 1: 'd'}, 1: 'b'}}}

def arbre_huffman(occurrences):
    """renvoie un dictionnaire avec le nombre d'occurences pour chacun des différents caractères rencontrés
    Parameters :
    ------------ dictionnaire contruit par la fonction table_frequences(texte) avec les caractères en clé et les nombres d'occurence en valeurs
    Returns:
    -------- table est un dictionnaire représentant l'arbre d'Huffman associé aux caractères rencontrés
    arbre_huffman(table_frequences('abracadabra')) pourra renvoyer {0: 'a', 1: {0: 'b', 1: {0: {0: 'd', 1: 'c'}, 1: 'r'}}}
    aux permutations près des lettres de même poids (d et c; b et r)
    ou bien un message "une seule lettre" si un seul caractère apparaît
    """
    tas = [(occ, lettre) for (lettre, occ) in occurrences.items()]
    if len(tas)==1:
        return "une seule lettre"
#    tas=sorted(tas,key=lambda x:x[1])  #ordre alphabétique pour unicité de l'arbre 
    tas=sorted(tas,key=lambda x:x[0])
    
    for i in range(len(tas)-1):
        # Là, pour avoir unicité de l'arbre, on peut choisir les lettres dans l'ordre alphabétique
        tas=sorted(tas,key=lambda x:x[0])        
        tampon=(tas[0][0]+tas[1][0],{0:tas[0][1],1:tas[1][1]})
        tas.pop(0)
        tas.pop(0)
        tas.append(tampon)
        tas[0],tas[len(tas)-1]=tas[len(tas)-1],tas[0]
    return tas[0][1]

#fonction code_huffman qui étant donné un arbre de Huffman construit par la fonction précédente retourne
# un dictionnaire où les clés sont les chaînes binaires et les valeurs les caractères correspondants.
#
#Le prototype de la fonction sera le suivant def code_huffman(arbre):
#
#et l’appel de C = code_huffman(T) ; print T
#
#avec T l’arbre précédent, arbre={0: 'a', 1: {0: 'b', 1: {0: {0: 'c', 1: 'd'}, 1: 'r'}}}
# devra retourner un dictionnaire de la forme suivante {'111': 'r', '1100': 'c', '10': 'b', '1101': 'd', '0': 'a'}

def code_huffman_parcours(arbre,prefixe,code):
    """modifie en place arbre, fonction recursive
    Parameters :
    ------------ arbre d'Huffman renvoyé par la fonction arbre_huffman(occurrences)
    """
    for noeud in arbre:
        if len(arbre[noeud]) == 1: 
            code[prefixe+str(noeud)] = arbre[noeud]
        else:
            code_huffman_parcours(arbre[noeud],prefixe+str(noeud),code)

def code_huffman(arbre):
    """modifie en place arbre, fonction recursive
    Parameters :
    ------------ arbre d'Huffman renvoyé par la fonction arbre_huffman(occurrences)
    Returns:
    -------- dictionnaire ayant pour clées les codes binaires et en valeurs les caractères
    """
    code = {}
    code_huffman_parcours(arbre,'',code)
    return code

#fonction encodage qui étant donné un code de Huffman construit par la fonction précédente et le texte initial retourne la chaîne de bits produite par le codage de Huffman.
#
#Le prototype de la fonction sera le suivant def encodage(code,texte):
#
#et l’appel de encodage(C,"abracadabra")
#
#avec C le code précédent avec arbre={0: 'a', 1: {0: 'b', 1: {0: {0: 'c', 1: 'd'}, 1: 'r'}}},
# devra retourner la chaîne binaire suivante '01011101100011010101110'
def encodage(texte,code):
    code_inv = dict((code[bits], bits) for bits in code)
    # construit le dictionnaire inverse
    texte_binaire = ''
    for c in texte:
        texte_binaire = texte_binaire + code_inv[c]
    return texte_binaire
# fonction decodage qui étant donnés un code de Huffman et un texte binaire compressé retourne le texte initial.
#
#Le prototype de la fonction sera le suivant def decodage(code,texte_binaire):
#
#et l’appel de decodage(T,'01011101100011010101110')avec arbre={0: 'a', 1: {0: 'b', 1: {0: {0: 'c', 1: 'd'}, 1: 'r'}}},
#
#avec C le code précédent, devra retourner la chaîne binaire suivante 'abracadabra'
def decodage(code,texte_binaire):
    texte = ''
    tampon = ''
    for b in texte_binaire:
        tampon = tampon+b
        if tampon in code:
            texte = texte+code[tampon]
            tampon = ''
    return texte



alphabetFrancais={'a':67563628,'b':10817171,'c':30219574,'d':34914685,
                  'e':131024205,'f':10579192,'g':11684140,'h':10583562,
                  'i':62672992,'j':3276064,'k':2747547,'l':47171247,
                  'm':24894034,'n':60728196,'o':47724400,'p':23647179,
                  'q':6140307,'r':57656209,'s':61882785,'t':56267109,
                  'u':42698875,'v':10590858,'w':1653435,'x':3588990,
                  'y':4351953,'z':1433913,' ':73751}
table_frequences(alphabetFrancais)   