import table_de_codage as tb

def frequenceCaracteres(texte):
    """Détermine la fréquence des caractères dans une chaine de caracteres
    Parametres:
    -----------
        texte : chaine de caracteres à analyser
    Retourne :
    --------
        un dictionnaire ('caractère', fréquence)
    """
    frequence = {}
    for c in texte :
        if (c in frequence):
            frequence[c] += 1
        else :
            frequence[c] = 1
    for cle, valeur in frequence.items() :
        frequence[cle] = round(valeur/len(texte)*100,1)
    return frequence

def codage(texte) :
    """Code une chaîne de caractères à partir d'une table de codage
    Parametres:
    -----------
        texte : chaîne de caractères à coder 
    Retourne :
    --------
        une chaîne de caractères contenant le message codé
    """
    code = ""
    for c in texte :
        code += tb.table[c]
    return code

def decodage(code):
    """Décode une chaîne de caractères à partir d'une table de codage
    Parametres:
    -----------
        texte : chaîne de caractères à décoder 
    Retourne :
    --------
        une chaîne de caractères contenant le message décodé
    """
    texte = ""
    debut = 0
    fin = 1
    while fin <= len(code) :
        mot = code[debut:fin]
        for cle, valeur in tb.table.items():
            if mot == valeur :
                texte += cle
                debut = fin
        fin += 1
    return texte

def compression(texte) :
    """Affiche le message d'origine, le nombre de bots sur lequel il est codé, puis le message codé et le nombre de bits sur lequel il est codé, puis le taux de compression
    Parametres:
    -----------
        texte : chaîne de caractères à coder 
    Retourne :
    --------
        Rien, nada, walou
    """
    print("Texte original : " ,texte, " (code sur ", len(texte)*8, " bits)")
    code = codage(texte)
    print("Code : ",code)
    print("Texte decode : ", decodage(code), " (code sur ", len(code), "bits)")
    print("Taux de compression : ", (len(texte)*8-len(code))/(len(texte)*8), "\n")

freq = frequenceCaracteres(tb.texte)
freq = sorted(freq.items(), key = lambda x : x[1]) #Trie les fréquences par ordre croissant
print(freq)

compression(tb.texte)

